﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyProject.Domain;
using NHibernate;
using NHibernate.Criterion;
using System.Threading;
using System.Configuration;

namespace MyProject.DomainPersistence
{
    public class Customers : ICustomerRepository
    {
        ISession _session;

        /// <summary>
        /// Initializes a new instance of the Customers class.
        /// </summary>
        /// <param name="session"></param>
        public Customers(ISession session)
        {
            _session = session;
        }

        public IEnumerable<Customer> GetAllCustomers()
        {
            SimulateLongRunningOperation();

            using (_session.BeginTransaction())
            {
                var crit = _session.CreateQuery("from Customer");

                return crit.List<Customer>();
            }
        }

        public IEnumerable<Customer> GetAllCustomersByState(string state)
        {
            SimulateLongRunningOperation();

            using (_session.BeginTransaction())
            {
                var crit = _session.CreateCriteria<Customer>().Add(Restrictions.Eq("State", state));

                return crit.List<Customer>();
            }
        }

        public Customer Get(int customerId)
        {
            SimulateLongRunningOperation();

            using (_session.BeginTransaction())
            {
                return _session.Get<Customer>(customerId);
            }

        }

        public void Put(Customer customer)
        {
            SimulateLongRunningOperation();

            using (var tx = _session.BeginTransaction())
            {
                try
                {
                    _session.SaveOrUpdate(customer);
                }
                catch (Exception e)
                {
                    tx.Rollback();
                }
                tx.Commit();
            }

        }

        private void SimulateLongRunningOperation()
        {
            var simulateDelay = ConfigurationManager.AppSettings.Get("simulatedDelay");

            if (!string.IsNullOrEmpty(simulateDelay) &&  simulateDelay.ToLower() == "true")
                Thread.Sleep(5000);
        }

    }
}
