﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using MyProject.Domain;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;
using System.Runtime.Serialization;
using Proteus.Utility.UnitTest;
using System.Runtime.Serialization.Formatters.Soap;

namespace MyProject.CustomersAndOrdersTests
{
    [TestFixture]
    public class SerializationTests : UnitTestBase
    {

        public class XmlCustomerFormatter : IFormatter
        {
            public SerializationBinder Binder
            {
                get
                {
                    throw new NotImplementedException();
                }
                set
                {
                    throw new NotImplementedException();
                }
            }

            public StreamingContext Context
            {
                get
                {
                    throw new NotImplementedException();
                }
                set
                {
                    throw new NotImplementedException();
                }
            }

            public ISurrogateSelector SurrogateSelector
            {
                get
                {
                    throw new NotImplementedException();
                }
                set
                {
                    throw new NotImplementedException();
                }
            }

            public object Deserialize(Stream serializationStream)
            {
                return new XmlSerializer(typeof(Customer)).Deserialize(serializationStream);
            }

            public void Serialize(Stream serializationStream, object graph)
            {
                new XmlSerializer(typeof(Customer)).Serialize(serializationStream, graph);
            }

        }

        private Customer BuildValidCustomer()
        {
            var customer = new Customer();
            var order = new Order();
            var item1 = new OrderItem() { Quantity = 1, Cost = 1 };
            var item2 = new OrderItem() { Quantity = 2, Cost = 2 };

            order.AddItem(item1);
            order.AddItem(item2);
            customer.AddOrder(order);

            SetInstanceFieldValue(customer, "_persistenceId", 1);

            return customer;
        }

        [Test]
        public void Can_Binary_Serialize_Customer()
        {
            var customer = BuildValidCustomer();
            var deserializedCustomer = DoSerializeDeserialize(new BinaryFormatter(), customer);

            Assert.AreEqual(customer, deserializedCustomer);
        }

        [Test]
        public void Can_Not_Xml_Serialize_Customer()
        {
            var customer = BuildValidCustomer();
            var deserializedCustomer = DoSerializeDeserialize(new XmlCustomerFormatter(), customer);

            Assert.AreNotEqual(customer, deserializedCustomer);
        }

        private Customer DoSerializeDeserialize(IFormatter serializer, Customer customer)
        {
            var stream = new MemoryStream();

            serializer.Serialize(stream, customer);

            stream.Position = 0;

            return serializer.Deserialize(stream) as Customer;
        }
    }
}
