﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using Proteus.Utility.UnitTest;
using MyProject.DomainPersistence;
using NHibernate.Cfg;
using NHibernate;
using MyProject.Domain;

namespace MyProject.PersistenceTests
{
    [TestFixture]
    public class CustomerRepositoryTests : DatabaseUnitTestBase
    {
        private ISession _session;

        private ISessionFactory _sesssionFactory;

        [TestFixtureSetUp]
        public void _TestFixtureSetUp()
        {
            _sesssionFactory = new Configuration().Configure().BuildSessionFactory();
            DatabaseFixtureSetUp();
        }

        [TestFixtureTearDown]
        public void _TestFixtureTearDown()
        {
            DatabaseFixtureTearDown();
        }

        [SetUp]
        public void _TestSetup()
        {
            _session = _sesssionFactory.OpenSession();
            DatabaseSetUp();
        }

        [TearDown]
        public void _TestTearDown()
        {
            DatabaseTearDown();
        }

        [Test]
        public void CanRetrieveAllCustomers()
        {
            var repo = new Customers(_session);
            var allTheCustomers = repo.GetAllCustomers();

            Assert.That(allTheCustomers, (Has.Count.EqualTo(3)));
        }

        [Test]
        public void CanStoreCustomer()
        {
            var repo = new Customers(_session);
            var newCustomer = new Customer() { Firstname = "Steve", Lastname = "Bohlen", State = "SpecialState" };

            repo.Put(newCustomer);

            var retrievedCustomer = repo.Get(newCustomer.Id);

            Assert.That(retrievedCustomer, Is.EqualTo(newCustomer));
        }


        [Test]
        public void CanGetCustomersByState()
        {
            var repo = new Customers(_session);
            var allTheCustomers = repo.GetAllCustomersByState("PA");

            Assert.That(allTheCustomers, Has.Count.EqualTo(2));
        }


    }
}
