using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using MyProject.Domain;
using MyProject.Services;
using Rhino.Mocks;

namespace MyProject.PlacingAndShippingOrdersTests
{
    [TestFixture]
    public class When_Shipping_Valid_Order
    {
        [Test]
        public void Can_Ship_Items()
        {
            var item1 = new OrderItem() { Cost = 1, Quantity = 1 };
            var item2 = new OrderItem() { Cost = 1, Quantity = 1 };

            var shippingItems = new List<ShippingItem>() { new ShippingItem(item1), new ShippingItem(item2) };

            var order = new Order();
            order.AddItem(item1);
            order.AddItem(item2);

            var customer = new Customer() { PostalCode = "12345" };
            customer.AddOrder(order);

            var approver = MockRepository.GenerateMock<IOrderApprovalService>();
            approver.Expect(a => a.IsValidToShip(customer, order)).IgnoreArguments().Return(true).Repeat.Any();

            var warehouse = MockRepository.GenerateMock<IWarehouseService>();
            warehouse.Expect(wh => wh.GetItemsFor(order)).Return(shippingItems);

            var dispatcher = MockRepository.GenerateMock<IShippingDispatcherService>();
            dispatcher.Expect(d => d.ShipTo(customer, shippingItems));

            var shipper = new OrderShipper(approver, warehouse, dispatcher);

            shipper.Ship(customer, order);

            dispatcher.VerifyAllExpectations();
            warehouse.VerifyAllExpectations();
            approver.VerifyAllExpectations();
        }
    }
}
